// Copyright (C) 1997, 1998 by DI Wolfgang Platzer, IAIK
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package org.w3c.jigsaw.http;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.io.*;

import org.w3c.tools.timers.*;

/**
 * This class extends the CommonLogger so that every 7 days new log and errlog files are
 * created. The filename looks like: DDMMYYY.log and DDMMYYYY.err.
 */
public class ExtendedLogger extends CommonLogger implements org.w3c.tools.timers.EventHandler {

  int default_period = 7;             // every 7 days new files
  long period;                        // the period in ms
  String log_name;                    // the name of the log file
  String errlog_name;                 // the name of the errlog file

  /**
   * Name of the property indicating the period for a log file.
   * This property indicates after how many days new log files shall be created.
   * <p>This property defaults to 7 days. 
   */
  public static final 
  String LOGPERIOD_P = "org.w3c.jigsaw.http.ExtendedLogger.period";

  /**
   * Handles a timer event which shall occur every 7 days. Simply closes the old
   * log files and creates new ones according to the current date.
   */
  public void handleTimerEvent(Object data, long time) {

    SimpleDateFormat formatter = new SimpleDateFormat("ddMMyyyy");
    String dateString = formatter.format(new Date());

    // don't use properties
    log_name = getFilename("x.x.x", dateString+".log");
    errlog_name = getFilename("x.x.x", dateString+".err");
    
    openLogFile();
    openErrorLogFile();
    
    if (server.timer.registerTimer(period, this, null) == null)
      server.errlog("ExtendedLogger: Unable to register a Timer.");
  }

  /**
   * Opens a new log file.
   */
  protected void openLogFile() {
  	try {
	    RandomAccessFile old = log ;
	    log = new RandomAccessFile(log_name, "rw");
	    log.seek (log.length());
	    if (old != null)
    		old.close();
  	} catch (IOException e) {
	    throw new HTTPRuntimeException (this.getClass().getName(), "openLogFile", "unable to open "+log_name);
  	}
  }

  /**
   * Opens a new errlog file.
   */
  protected void openErrorLogFile() {
  	try {
	    RandomAccessFile old = errlog;
	    errlog = new RandomAccessFile(errlog_name, "rw");
	    errlog.seek (errlog.length());
	    if (old != null)
    		old.close() ;
  	} catch (IOException e) {
	    throw new HTTPRuntimeException (this.getClass().getName(), "openErrorLogFile", "unable to open "+errlog_name);
  	}
  }  

  /**
   * Intitializes the ExtendedLogger by registering a timer event.
   */
  public void initialize(httpd server) {
  	this.server = server ;
	  props  = server.getProperties() ;

	  int days = props.getInteger(LOGPERIOD_P, default_period);
    period = days*24*60*60*1000;

    handleTimerEvent(null, 0);    
    super.initialize(server);
  }

  /**
   * The constructor.
   */
  ExtendedLogger() {
    super();
  }
}